package me.km.utils;

import java.lang.reflect.Field;
import net.minecraft.enchantment.Enchantment;
import net.minecraft.entity.passive.EntityVillager;
import net.minecraft.entity.projectile.EntityArrow;
import net.minecraft.init.Enchantments;
import net.minecraft.init.SoundEvents;
import net.minecraft.util.FoodStats;
import net.minecraft.util.SoundEvent;

public class ReflectionUtils 
{    
    // -----------------------------------------------------------------------------------
    // helper stuff
    // -----------------------------------------------------------------------------------
    
    private static Field getField(Class c, String field)
    {
        try
        {
            Field f = c.getDeclaredField(field);
            f.setAccessible(true);
            return f;
        }
        catch(NoSuchFieldException | SecurityException ex)
        {
            System.out.println(field + " - " + ex);
        }
        return null;
    }
    
    private static <T> void setInt(Class<T> c, T t, String field, int i)
    {
        Field f = getField(c, field);
        try
        {
            f.setInt(t, i);
        }
        catch(SecurityException | IllegalArgumentException | IllegalAccessException | NullPointerException ex)
        {
            System.out.println(field + " - " + ex);
        }
    }
    
    private static <T> int getInt(Class<T> c, T t, String field, int error)
    {
        Field f = getField(c, field);
        try
        {
            f.setAccessible(true);
            return f.getInt(t);
        }
        catch(SecurityException | IllegalArgumentException | IllegalAccessException | NullPointerException ex)
        {
            System.out.println(field + " - " + ex);
            return error;
        }
    }
    
    private static <T> void setFloat(Class<T> c, T t, String field, float fl)
    {
        Field f = getField(c, field);
        try
        {
            f.setFloat(t, fl);
        }
        catch(SecurityException | IllegalArgumentException | IllegalAccessException | NullPointerException ex)
        {
            System.out.println(field + " - " + ex);
        }
    }
    
    private static <T> float getFloat(Class<T> c, T t, String field, float error)
    {
        Field f = getField(c, field);
        try
        {
            f.setAccessible(true);
            return f.getFloat(t);
        }
        catch(SecurityException | IllegalArgumentException | IllegalAccessException | NullPointerException ex)
        {
            System.out.println(field + " - " + ex);
            return error;
        }
    }
    
    private static <T> T getFieldValue(Class<T> cast, Class c, String field)
    {
        try
        {
            return (T) c.getField(field).get(null);
        }
        catch(NoSuchFieldException | SecurityException | IllegalAccessException | IllegalArgumentException ex)
        {
            return null;
        }
    }
    
    // -----------------------------------------------------------------------------------
    // Villager stuff
    // -----------------------------------------------------------------------------------
    
    public static void setCareerLevel(EntityVillager v, int level)
    {
        setInt(EntityVillager.class, v, "careerLevel", level);
    }
    
    // -----------------------------------------------------------------------------------
    // FoodStats
    // -----------------------------------------------------------------------------------
    
    public static void setExhaustion(FoodStats stats, float f)
    {
        setFloat(FoodStats.class, stats, "foodExhaustionLevel", f);
    }
    
    public static float getExhaustion(FoodStats stats)
    {
        return getFloat(FoodStats.class, stats, "foodExhaustionLevel", 0);
    }
    
    public static void setSaturation(FoodStats stats, float f)
    {
        setFloat(FoodStats.class, stats, "foodSaturationLevel", f);
    }
    
    public static float getSaturation(FoodStats stats)
    {
        return getFloat(FoodStats.class, stats, "foodSaturationLevel", 0);
    }
    
    // -----------------------------------------------------------------------------------
    // field list gets
    // -----------------------------------------------------------------------------------
    
    public static Enchantment getEnchantment(String name)
    {
        return getFieldValue(Enchantment.class, Enchantments.class, name);
    }
    
    public static SoundEvent getSoundEvent(String name)
    {
        return getFieldValue(SoundEvent.class, SoundEvents.class, name);
    }
    
    private final static Field TIME_IN_GROUND = getField(EntityArrow.class, "timeInGround");
    
    public static int getArrowTimeInGround(EntityArrow arrow)
    {
        try
        {
            return TIME_IN_GROUND.getInt(arrow);
        }
        catch(IllegalAccessException | IllegalArgumentException | NullPointerException ex)
        {
            System.out.println("timeInGround" + " - " + ex);
            return 0;
        }
    }
}
